"""
Photo-Import
~~~~~~
A tool for importing photos from one directory into a hierarchical folder
structure in another directory based on the EXIF data of the photos.

:copyright: Copyright 2019 Edward Armitage.
:license: MIT, see LICENSE for details.
"""
from ._version import get_versions

__version__ = get_versions()["version"]
del get_versions



"""Photo Import.

A tool for importing photos from one directory into a hierarchical folder
structure in another directory based on the EXIF data of the photos.

Usage:
    photo-import [options] SOURCE DESTINATION

Arguments:
    SOURCE      photo to be imported
    DESTINATION directory to move photos into

Options:
    -h --help           Show this help.
    -s --silent         Silent mode. No logs will be output to stdout. Not
                        compatible with --dry-run.
    -d --dry-run        Dry-run-only mode. No directories will be created and
                        no files will be moved.
    -m --month-only     Only create folders for months, not individual days.
    -v --version        Show version.

"""
from pathlib import Path

from docopt import docopt

from photoimport import __version__
from photoimport.folders import FileMover, find_all_associated_files
from photoimport.photos import read_date
from photoimport.ui import ConsoleWriter


def main():
    """Main entry-point for the photo-import command.

    Processes all provided arguments, and performs the desired actions before
    exiting.
    """
    arguments = docopt(__doc__, version=__version__)

    if arguments["--silent"] and arguments["--dry-run"]:
        ConsoleWriter.fatal(-1, "--silent flag can't be used with --dry-run mode")

    writer = ConsoleWriter(arguments["--silent"])
    mover = FileMover(writer, arguments["DESTINATION"], arguments["--dry-run"], arguments["--month-only"])

    date = read_date(Path(arguments["SOURCE"]))
    files = find_all_associated_files(Path(arguments["SOURCE"]))

    mover.create_directory(date)
    for file in files:
        mover.move_file(file, date)



"""All the functionality to do with the filesystem

:copyright: Copyright 2019 Edward Armitage.
:license: MIT, see LICENSE for details.
"""
import os
import shutil
from datetime import date
from pathlib import Path
from typing import List

from photoimport.ui import ConsoleWriter


class FileMover:
    """Class for moving files into a date-organised hierarchy"""

    def __init__(self, writer: ConsoleWriter, base_path: str, dry_run_mode: bool, month_only: bool) -> None:
        """
        :param writer: The ConsoleWriter used to inform of progress
        :param base_path: The directory to build the date-organised hierarchy within
        :param dry_run_mode When true, no folders will be created on the filesystem and no files will
                be actually moved
        :param month_only: Whether to store all files for a given month in the same folder, rather
                than separating by day (default is False)
        """
        self._writer = writer
        self._base_path = Path(base_path)
        self._dry_run_mode = dry_run_mode
        self._month_only = month_only

        if self._dry_run_mode:
            self._writer.status("In dry-run mode. No folders will be created or files moved.")

    def create_directory(self, folder_date: date) -> None:
        """Creates a directory for the given date

        If the required folder does not already exist, it will be created (if not in dry-run
        mode) and log when not in silent mode. If a top-level folder already exists (e.g. the
        year or month folder), the lower level (e.g. month or date) folders will be created
        alongside any existing files.

        :param folder_date: The date of the folder to be created
        """
        directory = self._build_path(folder_date)

        if not os.path.isdir(directory):
            self._writer.action("📂 Creating {}".format(directory))
            if not self._dry_run_mode:
                os.makedirs(directory, exist_ok=True)

    def move_file(self, file: Path, folder_date: date) -> None:
        """Moves a file into an appropriate directory for the given date

        Will only move files if not in dry-run mode. Will log when not in silent
        mode.

        :param file: The file to be moved
        :param folder_date: The date of the folder to move the file into
        """
        directory = self._build_path(folder_date)
        self._writer.action("➡️  Moving {} to {}".format(file, directory))
        if not self._dry_run_mode:
            shutil.move(str(file), directory)

    def _build_path(self, folder_date: date) -> str:
        year = "{:04d}".format(folder_date.year)
        month = "{:02d}".format(folder_date.month)
        day = "{:02d}".format(folder_date.day)

        if self._month_only:
            path = os.path.join(str(self._base_path), year, month)
        else:
            path = os.path.join(str(self._base_path), year, month, day)

        return path


def find_all_associated_files(file: Path) -> List[Path]:
    """Finds the companion files alongside the provided the file in a given directory

    A companion file is a file within the same directory as the original file, with the same
    name (ignoring file extensions). For example photo-001.raw is a companion to photo-001.jpg,
    but photo-0011.jpg is not.

    :param file: The file to find companions for
    :return: a list of Paths containing the provided file and all companion files
    """
    return list(file.parent.glob("{}.*".format(file.stem)))



"""Everything required to print the UI for the user

:copyright: Copyright 2019 Edward Armitage.
:license: MIT, see LICENSE for details.
"""
import sys
from textwrap import indent

from colorama import init, Fore, Style


class ConsoleWriter:
    """Class for grouping UI functionality"""

    def __init__(self, silent_mode: bool) -> None:
        """
        :param silent_mode: When True, status() and action() are effectively
                no-op methods.
        """
        init(autoreset=True)
        self._silent_mode = silent_mode

    @staticmethod
    def fatal(error_code: int, msg: str) -> None:
        """Prints an error message, and exits with the provided error code

        :param error_code: The error code to use when exiting
        :param msg: The error message to be shown
        """
        print(Fore.RED + "❌ A fatal error occurred:", file=sys.stderr)
        print(indent(Fore.RED + msg, "    "), file=sys.stderr)
        print(Fore.RED + "Exiting" + Style.RESET_ALL, file=sys.stderr)
        exit(error_code)

    def status(self, msg: str) -> None:
        """Prints a message indicating the current status

        :param msg: The message to be printed
        """
        if not self._silent_mode:
            print(Fore.YELLOW + msg)

    def action(self, msg: str) -> None:
        """Prints a message showing an action that is being performed

        :param msg: The message to be printed
        """
        if not self._silent_mode:
            print(Fore.GREEN + msg)



from setuptools import setup

import versioneer

with open("README.rst", "rb") as f:
    readme = f.read().decode("utf-8")

setup(
    name="photo-import",
    packages=["photoimport"],
    entry_points={
        "console_scripts": ['photo-import = photoimport.command:main']
    },
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="Application to import photos into a hierarchical structure based on the EXIF data of the photos.",
    long_description=readme,
    long_description_content_type="text/x-rst",
    author="Edd Armitage",
    author_email="edward.armitage@gmail.com",
    url="https://gitlab.com/eddarmitage/photo-import",
    license="MIT",
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Environment :: Console',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    python_requires='>=3.4',
    install_requires=["exif", "colorama", "docopt"],
    setup_requires=[],
    tests_require=["pyfakefs", "nose2", "behave", "parse"],
    extras_require={
        'python_version<3.5': ['typing==3.7.4.1', ],
    },
)
